/*-------------------------------------------------------------------------
 *
 * int1--1.0.sql
 *	  PostgreSQL extension definitions for INT1
 *
 * Created by Randolf Richardson, 2017-Jul-20 through 2017-Jul-27
 * http://www.randolfrichardson.com/
 *
 * Portions Copyright (c) 2017, Randolf Richardson
 * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *	  contrib/int1/int1--1.0.sql
 *
 * VERSION HISTORY
 *        2017-Aug-02 v1.01 - Added support for GENERATE_SERIES
 *        2017-Jul-27 v1.00 - Initial version
 *
 *-------------------------------------------------------------------------
 */

-- complain if script is sourced in psql, rather than via CREATE EXTENSION
\echo Use "CREATE EXTENSION int1" to load this file. \quit

-- Example:
--   CREATE TABLE test (id INT1);
--   INSERT INTO test VALUES(127);
--
--   SELECT 127::INT1;
--   SELECT -1::INT1;
--

-- ----------------------------------------------------------------------
-- Type definitions
--
-- type creation is needlessly chatty

CREATE TYPE int1;

-- ----------------------------------------------------------------------
-- Input/Output subroutines

CREATE FUNCTION int1_in(cstring)    RETURNS int1    AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_out(int1)      RETURNS cstring AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_recv(internal) RETURNS int1    AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_send(int1)     RETURNS bytea   AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;

-- ----------------------------------------------------------------------
-- Type implementation

CREATE TYPE int1 (
       INPUT = int1_in, OUTPUT = int1_out,
       RECEIVE = int1_recv, SEND = int1_send,
       CATEGORY = 'N',
       INTERNALLENGTH = 1, -- PASSEDBYVALUE
       ALIGNMENT = char
);
COMMENT ON TYPE int1 IS 'Signed 8-bit data type';

-- ----------------------------------------------------------------------
-- Cast subroutines

CREATE FUNCTION int1_cast_from_bool(bool) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_bool(int1)   RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(bool AS int1) WITH FUNCTION int1_cast_from_bool(bool) AS ASSIGNMENT;
    CREATE CAST(int1 AS bool) WITH FUNCTION int1_cast_to_bool(int1)   AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_bytea(bytea) RETURNS int1  AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_bytea(int1)    RETURNS bytea AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(bytea AS int1) WITH FUNCTION int1_cast_from_bytea(bytea) AS ASSIGNMENT;
    CREATE CAST(int1 AS bytea) WITH FUNCTION int1_cast_to_bytea(int1)    AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_char(char) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_char(int1)   RETURNS char AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(char AS int1) WITH FUNCTION int1_cast_from_char(char) AS ASSIGNMENT;
    CREATE CAST(int1 AS char) WITH FUNCTION int1_cast_to_char(int1)   AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_float4(float4) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_float4(int1)   RETURNS float4 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(float4 AS int1) WITH FUNCTION int1_cast_from_float4(float4) AS ASSIGNMENT;
    CREATE CAST(int1 AS float4) WITH FUNCTION int1_cast_to_float4(int1)     AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_float8(float8) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_float8(int1)   RETURNS float8 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(float8 AS int1) WITH FUNCTION int1_cast_from_float8(float8) AS ASSIGNMENT;
    CREATE CAST(int1 AS float8) WITH FUNCTION int1_cast_to_float8(int1)     AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_int2(int2) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_int2(int1)   RETURNS int2 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(int2 AS int1) WITH FUNCTION int1_cast_from_int2(int2) AS ASSIGNMENT;
    CREATE CAST(int1 AS int2) WITH FUNCTION int1_cast_to_int2(int1)   AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_int4(int4) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_int4(int1)   RETURNS int4 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(int4 AS int1) WITH FUNCTION int1_cast_from_int4(int4) AS ASSIGNMENT;
    CREATE CAST(int1 AS int4) WITH FUNCTION int1_cast_to_int4(int1)   AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_int8(int8) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_int8(int1)   RETURNS int8 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(int8 AS int1) WITH FUNCTION int1_cast_from_int8(int8) AS ASSIGNMENT;
    CREATE CAST(int1 AS int8) WITH FUNCTION int1_cast_to_int8(int1)   AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_numeric(numeric) RETURNS int1    AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_numeric(int1)      RETURNS numeric AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(numeric AS int1) WITH FUNCTION int1_cast_from_numeric(numeric) AS ASSIGNMENT;
    CREATE CAST(int1 AS numeric) WITH FUNCTION int1_cast_to_numeric(int1)      AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_varbit(varbit) RETURNS int1   AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_varbit(int1)     RETURNS varbit AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(varbit AS int1) WITH FUNCTION int1_cast_from_varbit(varbit) AS ASSIGNMENT;
    CREATE CAST(int1 AS varbit) WITH FUNCTION int1_cast_to_varbit(int1)     AS ASSIGNMENT;

CREATE FUNCTION int1_cast_from_bit(bit) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_cast_to_bit(int1)  RETURNS bit  AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE CAST(bit AS int1) WITH FUNCTION int1_cast_from_bit(bit) AS ASSIGNMENT;
    CREATE CAST(int1 AS bit) WITH FUNCTION int1_cast_to_bit(int1)  AS ASSIGNMENT;

-- ----------------------------------------------------------------------
-- Unary Operator subroutines

CREATE FUNCTION int1_uminus(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR - (
       PROCEDURE = int1_uminus,
       RIGHTARG  = int1
);
CREATE FUNCTION int1_uplus(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR + (
       PROCEDURE = int1_uplus,
       RIGHTARG  = int1
);
CREATE FUNCTION int1_uflip(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR ~ (
       PROCEDURE = int1_uflip,
       RIGHTARG  = int1
);
CREATE FUNCTION int1_ureverse(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR ~@ (
       PROCEDURE = int1_ureverse,
       RIGHTARG  = int1
);
CREATE FUNCTION int1_abs(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR @ (
       PROCEDURE = int1_abs,
       RIGHTARG  = int1
);

-- ----------------------------------------------------------------------
-- Comparison Operator subroutines

CREATE FUNCTION int1_eq(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR   = (
       PROCEDURE  = int1_eq,
       LEFTARG    = int1,
       RIGHTARG   = int1,
       COMMUTATOR = !=
);
CREATE FUNCTION    int1_eq_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
CREATE FUNCTION    int1_eq_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
CREATE FUNCTION    int1_eq_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
CREATE FUNCTION  int1_eq_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
CREATE FUNCTION  int1_eq_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
CREATE FUNCTION int1_eq_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1, $2::int1);';
    CREATE OPERATOR = (PROCEDURE = int1_eq_int2,    LEFTARG = int1,    RIGHTARG = int2,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int1_eq_int4,    LEFTARG = int1,    RIGHTARG = int4,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int1_eq_int8,    LEFTARG = int1,    RIGHTARG = int8,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int1_eq_float4,  LEFTARG = int1,    RIGHTARG = float4,  COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int1_eq_float8,  LEFTARG = int1,    RIGHTARG = float8,  COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int1_eq_numeric, LEFTARG = int1,    RIGHTARG = numeric, COMMUTATOR = !=);
CREATE FUNCTION    int2_eq_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
CREATE FUNCTION    int4_eq_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
CREATE FUNCTION    int8_eq_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
CREATE FUNCTION  float4_eq_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
CREATE FUNCTION  float8_eq_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
CREATE FUNCTION numeric_eq_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_eq($1::int1, $2);';
    CREATE OPERATOR = (PROCEDURE = int2_eq_int1,    LEFTARG = int2,    RIGHTARG = int1,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int4_eq_int1,    LEFTARG = int4,    RIGHTARG = int1,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = int8_eq_int1,    LEFTARG = int8,    RIGHTARG = int1,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = float4_eq_int1,  LEFTARG = float4,  RIGHTARG = int1,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = float8_eq_int1,  LEFTARG = float8,  RIGHTARG = int1,    COMMUTATOR = !=);
    CREATE OPERATOR = (PROCEDURE = numeric_eq_int1, LEFTARG = numeric, RIGHTARG = int1,    COMMUTATOR = !=);

CREATE FUNCTION int1_ne(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR  != (
       PROCEDURE  = int1_ne,
       LEFTARG    = int1,
       RIGHTARG   = int1,
       COMMUTATOR = =
);
CREATE FUNCTION    int1_ne_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
CREATE FUNCTION    int1_ne_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
CREATE FUNCTION    int1_ne_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
CREATE FUNCTION  int1_ne_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
CREATE FUNCTION  int1_ne_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
CREATE FUNCTION int1_ne_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1, $2::int1);';
    CREATE OPERATOR != (PROCEDURE = int1_ne_int2,    LEFTARG = int1,    RIGHTARG = int2,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int1_ne_int4,    LEFTARG = int1,    RIGHTARG = int4,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int1_ne_int8,    LEFTARG = int1,    RIGHTARG = int8,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int1_ne_float4,  LEFTARG = int1,    RIGHTARG = float4,  COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int1_ne_float8,  LEFTARG = int1,    RIGHTARG = float8,  COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int1_ne_numeric, LEFTARG = int1,    RIGHTARG = numeric, COMMUTATOR = =);
CREATE FUNCTION    int2_ne_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
CREATE FUNCTION    int4_ne_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
CREATE FUNCTION    int8_ne_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
CREATE FUNCTION  float4_ne_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
CREATE FUNCTION  float8_ne_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
CREATE FUNCTION numeric_ne_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ne($1::int1, $2);';
    CREATE OPERATOR != (PROCEDURE = int2_ne_int1,    LEFTARG = int2,    RIGHTARG = int1,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int4_ne_int1,    LEFTARG = int4,    RIGHTARG = int1,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = int8_ne_int1,    LEFTARG = int8,    RIGHTARG = int1,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = float4_ne_int1,  LEFTARG = float4,  RIGHTARG = int1,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = float8_ne_int1,  LEFTARG = float8,  RIGHTARG = int1,    COMMUTATOR = =);
    CREATE OPERATOR != (PROCEDURE = numeric_ne_int1, LEFTARG = numeric, RIGHTARG = int1,    COMMUTATOR = =);

CREATE FUNCTION int1_gt(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR   > (
       PROCEDURE  = int1_gt,
       LEFTARG    = int1,
       RIGHTARG   = int1,
       COMMUTATOR = <
);
CREATE FUNCTION    int1_gt_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
CREATE FUNCTION    int1_gt_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
CREATE FUNCTION    int1_gt_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
CREATE FUNCTION  int1_gt_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
CREATE FUNCTION  int1_gt_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
CREATE FUNCTION int1_gt_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1, $2::int1);';
    CREATE OPERATOR > (PROCEDURE = int1_gt_int2,    LEFTARG = int1,    RIGHTARG = int2,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int1_gt_int4,    LEFTARG = int1,    RIGHTARG = int4,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int1_gt_int8,    LEFTARG = int1,    RIGHTARG = int8,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int1_gt_float4,  LEFTARG = int1,    RIGHTARG = float4,  COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int1_gt_float8,  LEFTARG = int1,    RIGHTARG = float8,  COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int1_gt_numeric, LEFTARG = int1,    RIGHTARG = numeric, COMMUTATOR = <);
CREATE FUNCTION    int2_gt_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
CREATE FUNCTION    int4_gt_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
CREATE FUNCTION    int8_gt_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
CREATE FUNCTION  float4_gt_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
CREATE FUNCTION  float8_gt_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
CREATE FUNCTION numeric_gt_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_gt($1::int1, $2);';
    CREATE OPERATOR > (PROCEDURE = int2_gt_int1,    LEFTARG = int2,    RIGHTARG = int1,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int4_gt_int1,    LEFTARG = int4,    RIGHTARG = int1,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = int8_gt_int1,    LEFTARG = int8,    RIGHTARG = int1,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = float4_gt_int1,  LEFTARG = float4,  RIGHTARG = int1,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = float8_gt_int1,  LEFTARG = float8,  RIGHTARG = int1,    COMMUTATOR = <);
    CREATE OPERATOR > (PROCEDURE = numeric_gt_int1, LEFTARG = numeric, RIGHTARG = int1,    COMMUTATOR = <);

CREATE FUNCTION int1_ge(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR  >= (
       PROCEDURE  = int1_ge,
       LEFTARG    = int1,
       RIGHTARG   = int1
);
CREATE FUNCTION    int1_ge_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
CREATE FUNCTION    int1_ge_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
CREATE FUNCTION    int1_ge_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
CREATE FUNCTION  int1_ge_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
CREATE FUNCTION  int1_ge_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
CREATE FUNCTION int1_ge_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1, $2::int1);';
    CREATE OPERATOR >= (PROCEDURE = int1_ge_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR >= (PROCEDURE = int1_ge_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR >= (PROCEDURE = int1_ge_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR >= (PROCEDURE = int1_ge_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR >= (PROCEDURE = int1_ge_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR >= (PROCEDURE = int1_ge_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_ge_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
CREATE FUNCTION    int4_ge_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
CREATE FUNCTION    int8_ge_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
CREATE FUNCTION  float4_ge_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
CREATE FUNCTION  float8_ge_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
CREATE FUNCTION numeric_ge_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_ge($1::int1, $2);';
    CREATE OPERATOR >= (PROCEDURE = int2_ge_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR >= (PROCEDURE = int4_ge_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR >= (PROCEDURE = int8_ge_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR >= (PROCEDURE = float4_ge_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR >= (PROCEDURE = float8_ge_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR >= (PROCEDURE = numeric_ge_int1, LEFTARG = numeric, RIGHTARG = int1);

CREATE FUNCTION int1_lt(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR   < (
       PROCEDURE  = int1_lt,
       LEFTARG    = int1,
       RIGHTARG   = int1,
       COMMUTATOR = >
);
CREATE FUNCTION    int1_lt_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
CREATE FUNCTION    int1_lt_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
CREATE FUNCTION    int1_lt_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
CREATE FUNCTION  int1_lt_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
CREATE FUNCTION  int1_lt_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
CREATE FUNCTION int1_lt_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1, $2::int1);';
    CREATE OPERATOR < (PROCEDURE = int1_lt_int2,    LEFTARG = int1,    RIGHTARG = int2,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int1_lt_int4,    LEFTARG = int1,    RIGHTARG = int4,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int1_lt_int8,    LEFTARG = int1,    RIGHTARG = int8,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int1_lt_float4,  LEFTARG = int1,    RIGHTARG = float4,  COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int1_lt_float8,  LEFTARG = int1,    RIGHTARG = float8,  COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int1_lt_numeric, LEFTARG = int1,    RIGHTARG = numeric, COMMUTATOR = >);
CREATE FUNCTION    int2_lt_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
CREATE FUNCTION    int4_lt_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
CREATE FUNCTION    int8_lt_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
CREATE FUNCTION  float4_lt_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
CREATE FUNCTION  float8_lt_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
CREATE FUNCTION numeric_lt_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_lt($1::int1, $2);';
    CREATE OPERATOR < (PROCEDURE = int2_lt_int1,    LEFTARG = int2,    RIGHTARG = int1,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int4_lt_int1,    LEFTARG = int4,    RIGHTARG = int1,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = int8_lt_int1,    LEFTARG = int8,    RIGHTARG = int1,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = float4_lt_int1,  LEFTARG = float4,  RIGHTARG = int1,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = float8_lt_int1,  LEFTARG = float8,  RIGHTARG = int1,    COMMUTATOR = >);
    CREATE OPERATOR < (PROCEDURE = numeric_lt_int1, LEFTARG = numeric, RIGHTARG = int1,    COMMUTATOR = >);

CREATE FUNCTION int1_le(int1, int1) RETURNS bool AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR <= (
       PROCEDURE = int1_le,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_le_int2(int1, int2   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
CREATE FUNCTION    int1_le_int4(int1, int4   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
CREATE FUNCTION    int1_le_int8(int1, int8   ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
CREATE FUNCTION  int1_le_float4(int1, float4 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
CREATE FUNCTION  int1_le_float8(int1, float8 ) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
CREATE FUNCTION int1_le_numeric(int1, numeric) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1, $2::int1);';
    CREATE OPERATOR <= (PROCEDURE = int1_le_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR <= (PROCEDURE = int1_le_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR <= (PROCEDURE = int1_le_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR <= (PROCEDURE = int1_le_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR <= (PROCEDURE = int1_le_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR <= (PROCEDURE = int1_le_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_le_int1(int2,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
CREATE FUNCTION    int4_le_int1(int4,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
CREATE FUNCTION    int8_le_int1(int8,    int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
CREATE FUNCTION  float4_le_int1(float4,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
CREATE FUNCTION  float8_le_int1(float8,  int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
CREATE FUNCTION numeric_le_int1(numeric, int1) RETURNS bool STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT int1_le($1::int1, $2);';
    CREATE OPERATOR <= (PROCEDURE = int2_le_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR <= (PROCEDURE = int4_le_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR <= (PROCEDURE = int8_le_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR <= (PROCEDURE = float4_le_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR <= (PROCEDURE = float8_le_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR <= (PROCEDURE = numeric_le_int1, LEFTARG = numeric, RIGHTARG = int1);

-- ----------------------------------------------------------------------
-- Bitwise Operator subroutines

CREATE FUNCTION int1_and(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR  & (
       PROCEDURE = int1_and,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_and_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    & $2);';
CREATE FUNCTION    int1_and_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    & $2);';
CREATE FUNCTION    int1_and_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    & $2::int4);';
    CREATE OPERATOR & (PROCEDURE = int1_and_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR & (PROCEDURE = int1_and_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR & (PROCEDURE = int1_and_int8,    LEFTARG = int1,    RIGHTARG = int8);
CREATE FUNCTION    int2_and_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 & $2::int2   );';
CREATE FUNCTION    int4_and_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 & $2::int4   );';
CREATE FUNCTION    int8_and_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 & $2::int4   );';
    CREATE OPERATOR & (PROCEDURE = int2_and_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR & (PROCEDURE = int4_and_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR & (PROCEDURE = int8_and_int1,    LEFTARG = int8,    RIGHTARG = int1);

CREATE FUNCTION int1_or(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR  | (
       PROCEDURE = int1_or,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_or_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    | $2);';
CREATE FUNCTION    int1_or_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    | $2);';
CREATE FUNCTION    int1_or_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    | $2::int4);';
    CREATE OPERATOR | (PROCEDURE = int1_or_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR | (PROCEDURE = int1_or_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR | (PROCEDURE = int1_or_int8,    LEFTARG = int1,    RIGHTARG = int8);
CREATE FUNCTION    int2_or_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 | $2::int2   );';
CREATE FUNCTION    int4_or_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 | $2::int4   );';
CREATE FUNCTION    int8_or_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 | $2::int4   );';
    CREATE OPERATOR | (PROCEDURE = int2_or_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR | (PROCEDURE = int4_or_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR | (PROCEDURE = int8_or_int1,    LEFTARG = int8,    RIGHTARG = int1);

CREATE FUNCTION int1_xor(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR  # (
       PROCEDURE = int1_xor,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_xor_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    # $2);';
CREATE FUNCTION    int1_xor_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    # $2);';
CREATE FUNCTION    int1_xor_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    # $2::int4);';
    CREATE OPERATOR # (PROCEDURE = int1_xor_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR # (PROCEDURE = int1_xor_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR # (PROCEDURE = int1_xor_int8,    LEFTARG = int1,    RIGHTARG = int8);
CREATE FUNCTION    int2_xor_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 # $2::int2   );';
CREATE FUNCTION    int4_xor_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 # $2::int4   );';
CREATE FUNCTION    int8_xor_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 # $2::int4   );';
    CREATE OPERATOR # (PROCEDURE = int2_xor_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR # (PROCEDURE = int4_xor_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR # (PROCEDURE = int8_xor_int1,    LEFTARG = int8,    RIGHTARG = int1);

CREATE FUNCTION int1_shl(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR << (
       PROCEDURE = int1_shl,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_shl_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    << $2);';
CREATE FUNCTION    int1_shl_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    << $2);';
CREATE FUNCTION    int1_shl_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    << $2::int4);';
    CREATE OPERATOR << (PROCEDURE = int1_shl_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR << (PROCEDURE = int1_shl_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR << (PROCEDURE = int1_shl_int8,    LEFTARG = int1,    RIGHTARG = int8);
CREATE FUNCTION    int2_shl_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 << $2::int2   );';
CREATE FUNCTION    int4_shl_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 << $2::int4   );';
CREATE FUNCTION    int8_shl_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 << $2::int4   );';
    CREATE OPERATOR << (PROCEDURE = int2_shl_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR << (PROCEDURE = int4_shl_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR << (PROCEDURE = int8_shl_int1,    LEFTARG = int8,    RIGHTARG = int1);

CREATE FUNCTION int1_shr(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR >> (
       PROCEDURE = int1_shr,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_shr_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    >> $2);';
CREATE FUNCTION    int1_shr_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    >> $2);';
CREATE FUNCTION    int1_shr_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    >> $2::int4);';
    CREATE OPERATOR >> (PROCEDURE = int1_shr_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR >> (PROCEDURE = int1_shr_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR >> (PROCEDURE = int1_shr_int8,    LEFTARG = int1,    RIGHTARG = int8);
CREATE FUNCTION    int2_shr_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 >> $2::int2   );';
CREATE FUNCTION    int4_shr_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 >> $2::int4   );';
CREATE FUNCTION    int8_shr_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 >> $2::int4   );';
    CREATE OPERATOR >> (PROCEDURE = int2_shr_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR >> (PROCEDURE = int4_shr_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR >> (PROCEDURE = int8_shr_int1,    LEFTARG = int8,    RIGHTARG = int1);

CREATE FUNCTION int1_shru(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR >>> (
       PROCEDURE = int1_shru,
       LEFTARG   = int1,
       RIGHTARG  = int1
);

-- ----------------------------------------------------------------------
-- Mathematical Operator subroutines

CREATE FUNCTION int1_plus(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR + (
       PROCEDURE = int1_plus,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_plus_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    + $2);';
CREATE FUNCTION    int1_plus_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    + $2);';
CREATE FUNCTION    int1_plus_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    + $2);';
CREATE FUNCTION  int1_plus_float4(int1, float4 ) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float4  + $2);';
CREATE FUNCTION  int1_plus_float8(int1, float8 ) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float8  + $2);';
CREATE FUNCTION int1_plus_numeric(int1, numeric) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::numeric + $2);';
    CREATE OPERATOR + (PROCEDURE = int1_plus_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR + (PROCEDURE = int1_plus_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR + (PROCEDURE = int1_plus_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR + (PROCEDURE = int1_plus_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR + (PROCEDURE = int1_plus_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR + (PROCEDURE = int1_plus_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_plus_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::int2   );';
CREATE FUNCTION    int4_plus_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::int4   );';
CREATE FUNCTION    int8_plus_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::int8   );';
CREATE FUNCTION  float4_plus_int1(float4,  int1) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::float4 );';
CREATE FUNCTION  float8_plus_int1(float8,  int1) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::float8 );';
CREATE FUNCTION numeric_plus_int1(numeric, int1) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 + $2::numeric);';
    CREATE OPERATOR + (PROCEDURE = int2_plus_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR + (PROCEDURE = int4_plus_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR + (PROCEDURE = int8_plus_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR + (PROCEDURE = float4_plus_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR + (PROCEDURE = float8_plus_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR + (PROCEDURE = numeric_plus_int1, LEFTARG = numeric, RIGHTARG = int1);

CREATE FUNCTION int1_minus(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR - (
       PROCEDURE = int1_minus,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_minus_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    - $2);';
CREATE FUNCTION    int1_minus_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    - $2);';
CREATE FUNCTION    int1_minus_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    - $2);';
CREATE FUNCTION  int1_minus_float4(int1, float4 ) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float4  - $2);';
CREATE FUNCTION  int1_minus_float8(int1, float8 ) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float8  - $2);';
CREATE FUNCTION int1_minus_numeric(int1, numeric) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::numeric - $2);';
    CREATE OPERATOR - (PROCEDURE = int1_minus_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR - (PROCEDURE = int1_minus_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR - (PROCEDURE = int1_minus_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR - (PROCEDURE = int1_minus_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR - (PROCEDURE = int1_minus_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR - (PROCEDURE = int1_minus_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_minus_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::int2   );';
CREATE FUNCTION    int4_minus_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::int4   );';
CREATE FUNCTION    int8_minus_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::int8   );';
CREATE FUNCTION  float4_minus_int1(float4,  int1) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::float4 );';
CREATE FUNCTION  float8_minus_int1(float8,  int1) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::float8 );';
CREATE FUNCTION numeric_minus_int1(numeric, int1) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 - $2::numeric);';
    CREATE OPERATOR - (PROCEDURE = int2_minus_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR - (PROCEDURE = int4_minus_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR - (PROCEDURE = int8_minus_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR - (PROCEDURE = float4_minus_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR - (PROCEDURE = float8_minus_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR - (PROCEDURE = numeric_minus_int1, LEFTARG = numeric, RIGHTARG = int1);

CREATE FUNCTION int1_mul(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR * (
       PROCEDURE = int1_mul,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_mul_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    * $2);';
CREATE FUNCTION    int1_mul_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    * $2);';
CREATE FUNCTION    int1_mul_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    * $2);';
CREATE FUNCTION  int1_mul_float4(int1, float4 ) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float4  * $2);';
CREATE FUNCTION  int1_mul_float8(int1, float8 ) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float8  * $2);';
CREATE FUNCTION int1_mul_numeric(int1, numeric) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::numeric * $2);';
    CREATE OPERATOR * (PROCEDURE = int1_mul_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR * (PROCEDURE = int1_mul_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR * (PROCEDURE = int1_mul_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR * (PROCEDURE = int1_mul_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR * (PROCEDURE = int1_mul_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR * (PROCEDURE = int1_mul_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_mul_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::int2   );';
CREATE FUNCTION    int4_mul_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::int4   );';
CREATE FUNCTION    int8_mul_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::int8   );';
CREATE FUNCTION  float4_mul_int1(float4,  int1) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::float4 );';
CREATE FUNCTION  float8_mul_int1(float8,  int1) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::float8 );';
CREATE FUNCTION numeric_mul_int1(numeric, int1) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 * $2::numeric);';
    CREATE OPERATOR * (PROCEDURE = int2_mul_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR * (PROCEDURE = int4_mul_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR * (PROCEDURE = int8_mul_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR * (PROCEDURE = float4_mul_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR * (PROCEDURE = float8_mul_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR * (PROCEDURE = numeric_mul_int1, LEFTARG = numeric, RIGHTARG = int1);

CREATE FUNCTION int1_div(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR / (
       PROCEDURE = int1_div,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_div_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    / $2);';
CREATE FUNCTION    int1_div_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    / $2);';
CREATE FUNCTION    int1_div_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    / $2);';
CREATE FUNCTION  int1_div_float4(int1, float4 ) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float4  / $2);';
CREATE FUNCTION  int1_div_float8(int1, float8 ) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::float8  / $2);';
CREATE FUNCTION int1_div_numeric(int1, numeric) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::numeric / $2);';
    CREATE OPERATOR / (PROCEDURE = int1_div_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR / (PROCEDURE = int1_div_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR / (PROCEDURE = int1_div_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR / (PROCEDURE = int1_div_float4,  LEFTARG = int1,    RIGHTARG = float4);
    CREATE OPERATOR / (PROCEDURE = int1_div_float8,  LEFTARG = int1,    RIGHTARG = float8);
    CREATE OPERATOR / (PROCEDURE = int1_div_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_div_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::int2   );';
CREATE FUNCTION    int4_div_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::int4   );';
CREATE FUNCTION    int8_div_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::int8   );';
CREATE FUNCTION  float4_div_int1(float4,  int1) RETURNS float4  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::float4 );';
CREATE FUNCTION  float8_div_int1(float8,  int1) RETURNS float8  STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::float8 );';
CREATE FUNCTION numeric_div_int1(numeric, int1) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 / $2::numeric);';
    CREATE OPERATOR / (PROCEDURE = int2_div_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR / (PROCEDURE = int4_div_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR / (PROCEDURE = int8_div_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR / (PROCEDURE = float4_div_int1,  LEFTARG = float4,  RIGHTARG = int1);
    CREATE OPERATOR / (PROCEDURE = float8_div_int1,  LEFTARG = float8,  RIGHTARG = int1);
    CREATE OPERATOR / (PROCEDURE = numeric_div_int1, LEFTARG = numeric, RIGHTARG = int1);

CREATE FUNCTION int1_mod(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR % (
       PROCEDURE = int1_mod,
       LEFTARG   = int1,
       RIGHTARG  = int1
);
CREATE FUNCTION    int1_mod_int2(int1, int2   ) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int2    % $2);';
CREATE FUNCTION    int1_mod_int4(int1, int4   ) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int4    % $2);';
CREATE FUNCTION    int1_mod_int8(int1, int8   ) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::int8    % $2);';
CREATE FUNCTION int1_mod_numeric(int1, numeric) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1::numeric % $2);';
    CREATE OPERATOR % (PROCEDURE = int1_mod_int2,    LEFTARG = int1,    RIGHTARG = int2);
    CREATE OPERATOR % (PROCEDURE = int1_mod_int4,    LEFTARG = int1,    RIGHTARG = int4);
    CREATE OPERATOR % (PROCEDURE = int1_mod_int8,    LEFTARG = int1,    RIGHTARG = int8);
    CREATE OPERATOR % (PROCEDURE = int1_mod_numeric, LEFTARG = int1,    RIGHTARG = numeric);
CREATE FUNCTION    int2_mod_int1(int2,    int1) RETURNS int2    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 % $2::int2   );';
CREATE FUNCTION    int4_mod_int1(int4,    int1) RETURNS int4    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 % $2::int4   );';
CREATE FUNCTION    int8_mod_int1(int8,    int1) RETURNS int8    STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 % $2::int8   );';
CREATE FUNCTION numeric_mod_int1(numeric, int1) RETURNS numeric STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT ($1 % $2::numeric);';
    CREATE OPERATOR % (PROCEDURE = int2_mod_int1,    LEFTARG = int2,    RIGHTARG = int1);
    CREATE OPERATOR % (PROCEDURE = int4_mod_int1,    LEFTARG = int4,    RIGHTARG = int1);
    CREATE OPERATOR % (PROCEDURE = int8_mod_int1,    LEFTARG = int8,    RIGHTARG = int1);
    CREATE OPERATOR % (PROCEDURE = numeric_mod_int1, LEFTARG = numeric, RIGHTARG = int1);

-- ----------------------------------------------------------------------
-- Utility subroutines

CREATE FUNCTION int1_inc(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_dec(int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_larger(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION int1_smaller(int1, int1) RETURNS int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE FUNCTION to_hex(int1) RETURNS text AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;

-- ----------------------------------------------------------------------
-- Generate Series subroutines

CREATE FUNCTION generate_series(int1, int1) RETURNS setof int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE FUNCTION generate_series(int1, int4) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1::int4, $2);';
    CREATE FUNCTION generate_series(int4, int1) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1,       $2::int4);';
CREATE FUNCTION generate_series(int1, int1, int1) RETURNS setof int1 AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
    CREATE FUNCTION generate_series(int1, int1, int4) RETURNS setof int1 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1,       $2,       $3::int1);';
    CREATE FUNCTION generate_series(int1, int4, int1) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1::int4, $2,       $3::int4);';
    CREATE FUNCTION generate_series(int1, int4, int4) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1::int4, $2,       $3);';
    CREATE FUNCTION generate_series(int4, int1, int1) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1,       $2::int4, $3::int4);';
    CREATE FUNCTION generate_series(int4, int4, int1) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1,       $2,       $3::int4);';
    CREATE FUNCTION generate_series(int4, int1, int4) RETURNS setof int4 STRICT IMMUTABLE LANGUAGE SQL AS 'SELECT generate_series($1,       $2::int4, $3);';

-- ----------------------------------------------------------------------
-- Index support

CREATE FUNCTION int1_cmp(int1, int1) RETURNS integer AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR CLASS btree_int1_ops DEFAULT FOR TYPE int1 USING btree AS
       OPERATOR	1 < ,
       OPERATOR	2 <= ,
       OPERATOR	3 = ,
       OPERATOR	4 >= ,
       OPERATOR	5 > ,
       FUNCTION	1 int1_cmp(int1, int1);

CREATE FUNCTION int1_hash(int1) RETURNS integer AS 'MODULE_PATHNAME' LANGUAGE C IMMUTABLE STRICT;
CREATE OPERATOR CLASS hash_int1_ops DEFAULT FOR TYPE int1 USING hash AS
       OPERATOR	1 = ,
       FUNCTION	1 int1_hash(int1);
